
# q-expansion of newform 400.6.c.g, downloaded from the LMFDB on 04 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 400
weight = 6
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[351, [1, 0]], [101, [1, 0]], [177, [-1, 0]]]
aps_data = [[0, 0], [0, 11], [0, 0], [0, -142], [-777, 0], [0, 884], [0, 27], [1145, 0], [0, -1854], [4920, 0], [-1802, 0], [0, -13178], [-15123, 0], [0, -7844], [0, -6732], [0, 3414], [33960, 0], [47402, 0], [0, -13177], [7548, 0], [0, -59821], [75830, 0], [0, -46299], [30585, 0], [0, -104018], [-23898, 0], [0, 22636], [0, 60633], [7090, 0], [0, -128841], [0, 141338], [-80052, 0], [0, -32253], [394865, 0], [491400, 0], [-200402, 0], [0, 22942], [0, 336241], [0, 59748], [0, -60696], [-7995, 0], [-454798, 0], [428298, 0], [0, -835531], [0, -678318], [-31900, 0], [423673, 0], [0, -398204], [0, -1257612], [-203780, 0], [0, 823974], [-555960, 0], [523577, 0], [-113127, 0], [0, -872958], [0, 1646466], [-1788720, 0], [-1121402, 0], [0, 598312], [-1530498, 0], [0, 1797001], [0, 754494], [0, -1966267], [599298, 0], [0, -721366], [0, -102348], [-1310477, 0], [0, 804397], [0, -2883207], [-1273550, 0], [0, 2830614], [-981090, 0], [0, -4191052], [0, 3234554], [1390355, 0], [0, -1141974], [-3462990, 0], [0, -5940068], [-2274123, 0], [4295515, 0], [1797045, 0], [-257548, 0], [-2229102, 0], [0, -4205851], [352640, 0], [0, -1283619], [2103975, 0], [0, -825233], [4501452, 0], [0, 1442116], [0, 393348], [-9176970, 0], [0, 6605978], [-38052, 0], [6856700, 0], [0, -8200164], [-4065810, 0], [5284077, 0], [0, -2533829], [498752, 0], [0, 3002693], [0, 12737292], [0, 5970816], [9269055, 0], [3895348, 0], [0, -7294163], [0, -8728197], [0, -13096281], [-13016850, 0], [-9939973, 0], [0, 15643808], [0, 9337934], [0, 5066802], [13767020, 0], [-2070602, 0], [-17911398, 0], [0, 17141356], [0, -8487732], [0, -2454786], [-5915565, 0], [4330952, 0], [0, -9139846], [0, -4572288], [0, -15322149], [-7025477, 0], [7911252, 0], [15448540, 0], [23054370, 0], [0, 16290488], [0, 12827924], [-12453460, 0], [0, 26324706], [17499448, 0], [0, -3463808], [-12617523, 0], [-5708045, 0], [0, 12082704], [0, -13763572], [0, 8777382], [10204590, 0], [1173748, 0], [-19806198, 0], [0, 30672196], [0, 25551963], [9194020, 0], [-15690960, 0], [0, 1601114], [0, 16461267], [19673585, 0], [0, -36806784], [0, 26959612], [34733502, 0], [0, -21618749], [0, 4481628], [0, -33663892], [1031748, 0], [4101470, 0], [-7716030, 0], [0, -43845803], [-10021548, 0], [0, 27952968], [0, -23181111], [0, -1584352], [-34455177, 0], [0, 29359887], [0, -8939574], [17889898, 0], [0, -33051668]]
